### Analysis of a randomized complete block experiment with r blocks and v treatments,
### using a model with fixed block effects and random treatment effects

## The dataset

Treatment <- rep(1:4, 3) ; Treatment
Block <- c(rep(1, 4), rep(2, 4), rep(3, 4)) ; Block
Y <- c(104, 106, 109, 111, 121, 128, 132, 130, 88, 89, 92, 95) ; Y
r <- 3 # Number of replicates 
v <- 4 # Number of treatments

## Model fit

library(lme4)
Model <- lmer(Y ~ factor(Block) - 1 + (1 | factor(Treatment)))
summary(Model)


## Calculation of prediction interval for the difference between Treatments 1 and 2


sigma_G <- as.vector(attr(VarCorr(Model)$"factor(Treatment)", "stddev"))
sigma_E <- as.vector(attr(VarCorr(Model), "sc")) 
k <- sigma_G^2/(sigma_G^2 + sigma_E^2/r) # Shrinkage factor

stderr <- sqrt(2*k*sigma_E^2/r)
d <- (k^2*(r - 1)*(v - 1))/(k^2*(r + 3) - 2*k*(r + 1) + r)
df <- max(1, d) ; df
stderr_kr <- sqrt(2*k*sigma_E^2/r + 8*sigma_E^2*(1 - k)/((r - 1)*(v - 1))) ; stderr_kr

means <- tapply(Y, Treatment, mean) ; means
pred <- k*(means[1] - means[2]) ; pred


# Prediction interval using the Satterthwaite method

lower_sat <- pred - qt(0.975, df)*stderr ; lower_sat
upper_sat <- pred + qt(0.975, df)*stderr ; upper_sat


# Prediction interval using the Kenward and Roger method

lower_kr <- pred - qt(0.975, df)*stderr_kr ; lower_kr ;
upper_kr <- pred + qt(0.975, df)*stderr_kr ; upper_kr ;


# Bayesian credible interval

samp <- mcmcsamp(Model, n = 50000, saveb = TRUE)
ranef <- attr(samp, "ranef")
quantile(ranef[1,] - ranef[2,], probs = c(0.025, 0.975))

